"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const promises_1 = __importDefault(require("fs/promises"));
const globby_1 = __importDefault(require("globby"));
const path_1 = __importDefault(require("path"));
const untildify_1 = __importDefault(require("untildify"));
const chalk_1 = __importDefault(require("./chalk"));
const findPackage_1 = __importDefault(require("./findPackage"));
const programError_1 = __importDefault(require("./programError"));
/**
 * Gets all workspace filenames, or just the root workspace package file
 *
 * @param options the application options, used to determine which packages to return.
 * @returns tuple(pkgs, workspaces) containing the pkgs and workspace string arrays
 */
async function getAllPackages(options) {
    var _a, _b, _c;
    const cwd = options.cwd ? (0, untildify_1.default)(options.cwd) : './';
    const rootPackageFile = options.packageFile || (options.cwd ? path_1.default.join(cwd, 'package.json') : 'package.json');
    // Workspace package names
    // These will be used to filter out local workspace packages so they are not fetched from the registry.
    let workspacePackages = [];
    // Find the package file with globby.
    // When in workspaces mode, only include the root project package file when --root is used.
    let pkgs = (!options.workspaces && !((_a = options.workspace) === null || _a === void 0 ? void 0 : _a.length)) || options.root
        ? // convert Windows path to *nix path for globby
            globby_1.default.sync(rootPackageFile.replace(/\\/g, '/'), { ignore: ['**/node_modules/**'] })
        : [];
    // workspaces
    if (options.workspaces || ((_b = options.workspace) === null || _b === void 0 ? void 0 : _b.length)) {
        // use silent, otherwise there will be a duplicate "Checking" message
        const [pkgData] = await (0, findPackage_1.default)({ ...options, packageFile: rootPackageFile, loglevel: 'silent' });
        const rootPkg = typeof pkgData === 'string' ? JSON.parse(pkgData) : pkgData;
        const workspaces = Array.isArray(rootPkg.workspaces) ? rootPkg.workspaces : (_c = rootPkg.workspaces) === null || _c === void 0 ? void 0 : _c.packages;
        if (!workspaces) {
            (0, programError_1.default)(options, chalk_1.default.red(`workspaces property missing from package.json. --workspace${options.workspaces ? 's' : ''} only works when you specify a "workspaces" property in your package.json.`));
        }
        // build a glob from the workspaces
        // FIXME: the following workspaces check is redundant
        /* c8 ignore next */
        const workspacePackageGlob = (workspaces || []).map(workspace => path_1.default
            .join(cwd, workspace, 'package.json')
            // convert Windows path to *nix path for globby
            .replace(/\\/g, '/'));
        // e.g. [packages/a/package.json, ...]
        const workspacePackageFiles = [
            ...globby_1.default.sync(workspacePackageGlob, {
                ignore: ['**/node_modules/**'],
            }),
        ];
        // Get the package names from the package files.
        // If a package does not have a name, use the folder name.
        // These will be used to filter out local workspace packages so they are not fetched from the registry.
        workspacePackages = await Promise.all(workspacePackageFiles.map(async (file) => {
            const packageFile = await promises_1.default.readFile(file, 'utf-8');
            const pkg = JSON.parse(packageFile);
            return pkg.name || file.split('/').slice(-2)[0];
        }));
        // add workspace packages
        pkgs = [
            ...pkgs,
            ...(options.workspaces
                ? // --workspaces
                    workspacePackageFiles
                : // --workspace
                    workspacePackageFiles.filter(pkgFile => {
                        var _a;
                        /* ignore coverage on optional-chaining */
                        /* c8 ignore next */
                        return (_a = options.workspace) === null || _a === void 0 ? void 0 : _a.some(workspace => 
                        /* ignore coverage on optional-chaining */
                        /* c8 ignore next */
                        workspaces === null || workspaces === void 0 ? void 0 : workspaces.some(workspacePattern => pkgFile === path_1.default.join(cwd, path_1.default.dirname(workspacePattern), workspace, 'package.json')));
                    })),
        ];
    }
    return [pkgs, workspacePackages];
}
exports.default = getAllPackages;
//# sourceMappingURL=getAllPackages.js.map